@@ -269,46 +269,31 @@ class Test_add_handler(unittest.TestCase):
                           'name', None, 'pyramid')
 
     def test_add_handler_pattern_None_with_previous_route(self):
-        import pyramid
+        from pyramid.exceptions import ConfigurationError
         config = self._makeOne()
         config.add_route('name', ':def')
-        views = []
-        def dummy_add_view(**kw):
-            views.append(kw)
-        class DummyHandler(object):
-            def one(self): pass
-        config.add_view = dummy_add_view
-        config.add_route = None # shouldn't be called
-        config.add_handler('name', None, DummyHandler)
-        self.assertEqual(len(views), 1)
-        view = views[0]
-        self.assertEqual(view['view'], DummyHandler)
+        self.assertRaises(ConfigurationError, config.add_handler,
+                          'name', None, 'pyramid')
     
     def test_add_handler_explicit_action_lacking(self):
-        import pyramid
         config = self._makeOne()
-        config.add_route('name', ':def')
         views = []
         def dummy_add_view(**kw): views.append(kw)
         class DummyHandler(object):
             def one(self): pass
         config.add_view = dummy_add_view # shouldn't be called
-        config.add_route = None # shouldn't be called
-        config.add_handler('name', None, DummyHandler, action='two')
+        config.add_handler('name', ':def', DummyHandler, action='two')
         self.assertEqual(len(views), 0)
 
     def test_add_handler_explicit_action_and_extra_exposed(self):
-        import pyramid
         config = self._makeOne()
-        config.add_route('name', ':def')
         views = []
         def dummy_add_view(**kw): views.append(kw)
         class DummyHandler(object):
             def two(self): pass
             two.__exposed__ = [{'name':'one'}]
         config.add_view = dummy_add_view # shouldn't be called
-        config.add_route = None # shouldn't be called
-        config.add_handler('name', None, DummyHandler, action='two')
+        config.add_handler('name', ':def', DummyHandler, action='two')
         self.assertEqual(len(views), 1)
         view = views[0]
         self.assertEqual(view['view'], DummyHandler)
@@ -379,11 +364,11 @@ class Test_add_handler(unittest.TestCase):
         try:
             config.commit()
         except ConfigurationConflictError, why:
-            c1, c2, c3, c4 = self._conflictFunctions(why)
-            self.assertEqual(c1, 'test_conflict_add_handler')
-            self.assertEqual(c2, 'test_conflict_add_handler')
-            self.assertEqual(c3, 'test_conflict_add_handler')
-            self.assertEqual(c3, 'test_conflict_add_handler')
+            c = list(self._conflictFunctions(why))
+            self.assertEqual(c[0], 'test_conflict_add_handler')
+            self.assertEqual(c[1], 'test_conflict_add_handler')
+            self.assertEqual(c[2], 'test_conflict_add_handler')
+            self.assertEqual(c[3], 'test_conflict_add_handler')
         else: # pragma: no cover
             raise AssertionError
 
@@ -513,29 +498,8 @@ class TestHandlerDirective(unittest.TestCase):
                 return 'OK'
         self._callFUT(context, 'name', '/:action', Handler)
         actions = extract_actions(context.actions)
-        self.assertEqual(len(actions), 3)
-
-        route_action = actions[0]
-        route_discriminator = route_action['discriminator']
-        self.assertEqual(route_discriminator[:2], ('route', 'name'))
-        self._assertRoute('name', '/:action')
-
-        view_action = actions[1]
+        _execute_actions(actions)
         request_type = reg.getUtility(IRouteRequest, 'name')
-        view_discriminator = view_action['discriminator']
-        discrim = ('view', None, '', None, IView, None, None, None, 'name',
-                   'one', False, None, None, None)
-        self.assertEqual(view_discriminator[:14], discrim)
-        view_action['callable'](*view_action['args'], **view_action['kw'])
-
-        view_action = actions[2]
-        request_type = reg.getUtility(IRouteRequest, 'name')
-        view_discriminator = view_action['discriminator']
-        discrim = ('view', None, '', None, IView, None, None, None, 'name',
-                   'two', False, None, None, None)
-        self.assertEqual(view_discriminator[:14], discrim)
-        view_action['callable'](*view_action['args'], **view_action['kw'])
-
         wrapped = reg.adapters.lookup(
             (IViewClassifier, request_type, Interface), IView, name='')
         self.failUnless(wrapped)
@@ -583,3 +547,9 @@ def extract_actions(native):
         d['order'] = order
         L.append(d)
     return L
+
+def _execute_actions(actions):
+    for action in sorted(actions, key=lambda x: x['order']):
+        if 'callable' in action:
+            if action['callable']:
+                action['callable']()
